%% estimateMonthlyVAR.m

close all;
clear all;

%% Loading data from data folder.
D = load('data\Vintage2008M01_1.mat');

%% Creating database containing monthly data for VAR.
d = struct();
d.GROWTH_1_ = D.GDPH*400;
d.PIE_1_ = D.PCU*400;
d.RS_1_   = D.FFED;
d.UNR_1_ = D.LR;
d.BLT_1_ = 1/4*(D.FTCRE + D.FTCIL + D.FTCIS + D.FTCNMH);
d.IP = D.IP;

%% Prepare VAR.

% List of variables.
list = fieldnames(d);
ny = length(list);
% VAR order.
p = 4;

%% Construct dummy observations for BVAR.

% Estimate std devs of VAR variables.
x = db2tseries(d,list);


range = get(x,'nanstart') : get(x,'nanend');
nper = length(range);
stdevs = std(x(range));
% Create Litterman's priors.
% The BVAR.litterman(rho,mu,lambda,ny,p) has the following input arguments:
% * rho = 0 is a white noise prior, rho = 1 is a random walk prior,
% * mu is the weight on the prior (can be a vector of numbers in which case
% each variable gets different weight).
% * lambda between 0 and Inf: priors on the k-th lag coefficients get
% k^lambda times bigger weights. If lambda = 0, all lags are treated
% equally. The higher the lambda, the more the coefficients are pulled
% towards the prior (zero in this case).
dum1 = BVAR.litterman(1,1000*stdevs,1,ny,p);

% Estimate a p-th order VAR and a p-th order BVAR.
w1 = VAR();
w2 = VAR();


[w1,data1] = estimate(w1,d,list,range,'order',p);
[w2,data2] = estimate(w2,d,list,range,'order',p,'bvar',dum1);

fcastrange = range(end) + (1:36);
% Unconditional forecasts (balanced panel).
uf1 = forecast(w1,d,fcastrange,[]);
uf1 = dbextend(d,uf1.mean);
uf2 = forecast(w2,d,fcastrange,[]);
uf2 = dbextend(d,uf2.mean);
% Conditional forecasts (unbalanced panel).
cf1 = forecast(w1,d,fcastrange,d);
cf1 = dbextend(d,cf1.mean);
cf2 = forecast(w2,d,fcastrange,d);
cf2 = dbextend(d,cf2.mean);

plotrange = range(end) + (-24:36);
nextplot([3,2]);
for i = 1 : ny
   nextplot();
   name = list{i};
   h = plot(plotrange,[cf1.(name),cf2.(name),d.(name){fcastrange}]);
   set(h(3),'lineStyle','none','marker','s','markerFaceColor','none','markerEdgeColor','black');
   set(h(1:2),'marker','.');
   grid('on');
   highlight(range(end)+(-24:0));
   title(list{i},'interpreter','none');
   set(gcf,'Position',get(0,'Screensize'));
end
legend('Conditional VAR forecast','Conditional BVAR forecast','Tunes');
