%% runVARHFI.m

close all;
clear all;

readDefinitions;

%%
m = loadstruct('GPM_HFI.mat');
h = loadstruct('data.mat');

%% VAR order.
p = 4;

%% Loading data from data folder.
D = load('data\Vintage2008M01_1.mat');

%% Creating database containing monthly data for VAR.
d = struct();
d.GROWTH_= D.GDPH*400;
d.PIE_ = D.PCU*400;
d.RS_   = D.FFED;
d.UNR_ = D.LR;
d.BLT_ = 1/4*(D.FTCRE + D.FTCIL + D.FTCIS + D.FTCNMH);
d.IP = D.IP;

%% Prepare VAR.

% List of variables.
list = fieldnames(d);
ny = length(list);


%% Construct dummy observations for BVAR.

% Estimate std devs of VAR variables.
x = db2tseries(d,list);
range = get(x,'nanstart') : get(x,'nanend'); 
nper = length(range);
stdevs = std(x(range));
% Create Litterman's priors.
% The BVAR.litterman(rho,mu*std,lambda,ny,p) has the following input arguments:
% * rho = 0 is a white noise prior, rho = 1 is a random walk prior,
% * mu is the weight on the prior (can be a vector of numbers in which case
% each variable gets different weight).
% * lambda between 0 and Inf: priors on the k-th lag coefficients get
% k^lambda times bigger weights. If lambda = 0, all lags are treated
% equally. The higher the lambda, the more the coefficients are pulled
% towards the prior (zero in this case).
dummyobs = BVAR.litterman(1,1*stdevs,1,ny,p);


%% Estimate a p-th order BVAR.
w1 = VAR();
[w1,data2] = estimate(w1,d,list,range,'order',p,'bvar',dummyobs);

fcastrange = range(end) + (1:36);

%% Conditional forecasts (unbalanced panel).
cf1 = forecast(w1,d,fcastrange,d);

%% Converting data to quarterly
dq = dbfun(@(x) convert(x,'q'),d);
dp = dbfun(@(x) convert(x,'q'),cf1.mean);
ds = dbfun(@(x) convert(x,'q'),cf1.std);

%% Adding 1-step-ahead BVAR predictons to historical data
%% and calibrating std deviations of prediction errors 
%% based on estimated uncertainty
for i = 1 : length(list)-1;
    nn = strcat(list{i},'1_');
    nn1 = strcat('std_RES_',nn);
    last = get(dq.(list{i}),'end');
    dq.(nn) = tseries();
    dq.(nn)(last+1) = dp.(list{i})(last+1);
    m.(nn1) = ds.(list{i})(last+1);
end;

%% Making 2 parameterizations
m = set(m,'nalt',2);

%% Removing the influence of HFIs in first parameterization
m(1).std_RES_GROWTH_1_ = Inf; 
m(1).std_RES_PIE_1_ = Inf;
m(1).std_RES_RS_1_ = Inf;
m(1).std_RES_UNR_1_ = Inf;
m(1).std_RES_BLT_1_ = Inf;

%% Running Kalman filter
[m,smooth] = filter(m,dq,starthist:endhist+horizon);

%% Plotting the historical data and models with
%% and without HFIs

list = {'GROWTH','PIE','RS','UNR'};
for i = 1 : length(list);
    subplot(2,2,i);
    plot(endhist-12:endhist,[h.(strcat(list{i},'_')) smooth.mean.(list{i})]);
    title(list{i});
    if i == 1;
        legend({'Data', 'No HFI', 'HFI'},'Location','SouthWest');
    end;
    highlight(qq(2008,1));
end;
